/*
 * force_sensors.c
 *
 * Created: 02/04/2019 11:33:55
 *  Author: jon_e
 */ 
 #include "force_sensors.h"

 #ifndef CLEAR_SCREEN
 #define CLEAR_SCREEN gfx_mono_generic_draw_filled_rect(0,0,130,30,0);
#endif

static uint16_t force_sensor_min_value[NO_OF_SENSORS] = {0};
static uint16_t force_sensor_max_value[NO_OF_SENSORS] = {0};

#define ADC_RES_FSR						ADC_RES_8

#define HYSTERESIS_WIDTH_8_BIT			5
#define HYSTERESIS_WIDTH_12_BIT			112

#define LEVEL_ONE_8_BIT					15
#define LEVEL_ONE_12_BIT				10

#define MIN_FORCE_LEVEL_0_RANGE_8_BIT	2
#define MIN_FORCE_LEVEL_0_RANGE_12_BIT	20

#define DEFAULT_MIN_VALUE_12_BIT		220
#define DEFAULT_MAX_VALUE_12_BIT		4095

#define DEFAULT_MIN_VALUE_8_BIT			14
#define DEFAULT_MAX_VALUE_8_BIT			200

#define STORED_MIN_SENSOR_VALUE_ADR0	0
#define STORED_MIN_SENSOR_VALUE_ADR1	2
#define STORED_MIN_SENSOR_VALUE_ADR2	4

#define STORED_MAX_SENSOR_VALUE_ADR0	6
#define STORED_MAX_SENSOR_VALUE_ADR1	8
#define STORED_MAX_SENSOR_VALUE_ADR2	10

#define CALIBRATION_DATA_STORED_ADR		12 //Sets this to one to indicate that there is calibration data other than just garbage. 

static uint16_t force_level_limits[NO_OF_SENSORS][NO_OF_FORCE_LEVELS];
static uint16_t previouse_force_reading[NO_OF_SENSORS];
static uint8_t	previous_force_level[NO_OF_SENSORS];


static uint16_t DEFAULT_MAX_VALUE;
static uint16_t DEFAULT_MIN_VALUE;

static uint16_t MIN_FORCE_LEVEL_0_RANGE;

static uint16_t LEVEL_ONE;
static uint16_t HYSTERESIS_WIDTH;

static void force_sensors_save_to_eeprom(void){
	eeprom_write_word((uint16_t*)STORED_MIN_SENSOR_VALUE_ADR0,force_sensor_min_value[0]);
	eeprom_write_word((uint16_t*)STORED_MIN_SENSOR_VALUE_ADR1,force_sensor_min_value[1]);
	eeprom_write_word((uint16_t*)STORED_MIN_SENSOR_VALUE_ADR2,force_sensor_min_value[2]);

	eeprom_write_word((uint16_t*)STORED_MAX_SENSOR_VALUE_ADR0,force_sensor_max_value[0]);
	eeprom_write_word((uint16_t*)STORED_MAX_SENSOR_VALUE_ADR1,force_sensor_max_value[1]);
	eeprom_write_word((uint16_t*)STORED_MAX_SENSOR_VALUE_ADR2,force_sensor_max_value[2]);

	eeprom_write_word((uint16_t*)CALIBRATION_DATA_STORED_ADR,1);
}

static void force_sensors_read_from_eeprom(void){			
	force_sensor_min_value[0] = eeprom_read_word((uint16_t*)STORED_MIN_SENSOR_VALUE_ADR0);
	force_sensor_min_value[1] = eeprom_read_word((uint16_t*)STORED_MIN_SENSOR_VALUE_ADR1);
	force_sensor_min_value[2] = eeprom_read_word((uint16_t*)STORED_MIN_SENSOR_VALUE_ADR2);

	force_sensor_max_value[0] = eeprom_read_word((uint16_t*)STORED_MAX_SENSOR_VALUE_ADR0);
	force_sensor_max_value[1] = eeprom_read_word((uint16_t*)STORED_MAX_SENSOR_VALUE_ADR1);
	force_sensor_max_value[2] = eeprom_read_word((uint16_t*)STORED_MAX_SENSOR_VALUE_ADR2);
}

static void force_sensors_create_levels(void){
	for (uint8_t sensor_no = 0; sensor_no<NO_OF_SENSORS;sensor_no++){
		
		force_level_limits[sensor_no][0] = force_sensor_min_value[sensor_no] + LEVEL_ONE;
		
		uint16_t force_interval = force_sensor_max_value[sensor_no] - force_sensor_min_value[sensor_no];
				
		for(uint8_t force_level_no = 1; force_level_no<NO_OF_FORCE_LEVELS;force_level_no++){
			force_level_limits[sensor_no][force_level_no] = force_level_limits[sensor_no][force_level_no-1] + force_interval/(pow(2,(NO_OF_FORCE_LEVELS-force_level_no)));
			if (force_level_limits[sensor_no][force_level_no]> force_sensor_max_value[sensor_no]){
				force_level_limits[sensor_no][force_level_no] = force_sensor_max_value[sensor_no];
			}
		}
	}
	
}

static void force_sensors_calibrate(uint8_t calibrate){

	//If user said calibrate, check for max value when fingers are at rest.
	//Then, for each finger separately, apply max pressure, and record it.
	if (calibrate){
		uint16_t force_reading[NO_OF_SENSORS];

		//Calibrate for rest.
		gfx_mono_draw_string("Let all fingers",0,0,&sysfont);
		gfx_mono_draw_string("be at rest",0,10,&sysfont);
		delay_ms(1000);
		for(uint16_t i = 0;i<1000;i++){
			force_sensors_read(force_reading);
			for (int sensor_no = 0;sensor_no < NO_OF_SENSORS;sensor_no++){
				if (force_reading[sensor_no]> force_sensor_min_value[sensor_no]){
					force_sensor_min_value[sensor_no] = force_reading[sensor_no];
				}
			}
			delay_us(300);
		}

		delay_ms(1000);
		CLEAR_SCREEN;
		//endrest
	

		//Calibration for index finger.
		gfx_mono_draw_string("Put max pressure",0,0,&sysfont);
		gfx_mono_draw_string("on index finger",0,10,&sysfont);
		delay_s(1);
		
		for (uint16_t i = 0;i<1000;i++){
			force_sensors_read(force_reading);
			if (force_reading[0] > force_sensor_max_value[0]){
				force_sensor_max_value[0] = force_reading[0];
			}
			delay_us(300);
		}
		
		CLEAR_SCREEN;
		delay_s(1);
		//endindex

		//Calibration for middle finger
		gfx_mono_draw_string("Put max pressure",0,0,&sysfont);
		gfx_mono_draw_string("on middle finger",0,10,&sysfont);
		delay_s(1);
		
		for (uint16_t i = 0;i<1000;i++){
			force_sensors_read(force_reading);
			if (force_reading[1] > force_sensor_max_value[1]){
				force_sensor_max_value[1] = force_reading[1];
			}
			delay_us(300);
		}

		CLEAR_SCREEN;
		delay_s(1);
		//endmiddle

		//Calibration for thumb
		gfx_mono_draw_string("Put max pressure",0,0,&sysfont);
		gfx_mono_draw_string("on thumb",0,10,&sysfont);
		delay_s(1);
		
		for (uint16_t i = 0;i<1000;i++){
			force_sensors_read(force_reading);
			if (force_reading[2] > force_sensor_max_value[2]){
				force_sensor_max_value[2] = force_reading[2];
			}
			delay_us(300);
		}

		CLEAR_SCREEN;
		delay_s(1);
		//endthumb


		//Save the result to EEPROM
		force_sensors_save_to_eeprom();

		gfx_mono_draw_string("Calibration done,",0,0,&sysfont);
		gfx_mono_draw_string("and saved to EEPROM",0,10,&sysfont);
		delay_s(2);
		CLEAR_SCREEN;
		//endcalibration


	//If the user opted not to calibrate, checks if there is recorded calibration data in the EEPROM.
	//If there is no data written there, use default values.
	}else{
		if (eeprom_read_word((uint16_t*)CALIBRATION_DATA_STORED_ADR) == 1){
			force_sensors_read_from_eeprom();
		}else{
			for (int i = 0;i<NO_OF_SENSORS;i++){
					force_sensor_min_value[i] = DEFAULT_MIN_VALUE;
					force_sensor_max_value[i] = DEFAULT_MAX_VALUE;
				}
		}		
	}
	//endnocalibrate

	//Finally, create discrete force levels from the min and max.
	force_sensors_create_levels();	
}

void force_sensors_init(uint8_t calibrate){
	if(ADC_RES_FSR == ADC_RES_8){
		DEFAULT_MAX_VALUE		= DEFAULT_MAX_VALUE_8_BIT;
		DEFAULT_MIN_VALUE		= DEFAULT_MIN_VALUE_8_BIT;
		LEVEL_ONE				= LEVEL_ONE_8_BIT;
		HYSTERESIS_WIDTH		= HYSTERESIS_WIDTH_8_BIT;
		MIN_FORCE_LEVEL_0_RANGE = MIN_FORCE_LEVEL_0_RANGE_8_BIT;
		}else{
		DEFAULT_MAX_VALUE		= DEFAULT_MAX_VALUE_12_BIT;
		DEFAULT_MIN_VALUE		= DEFAULT_MIN_VALUE_12_BIT;
		LEVEL_ONE				= LEVEL_ONE_12_BIT;
		HYSTERESIS_WIDTH		= HYSTERESIS_WIDTH_12_BIT;
		MIN_FORCE_LEVEL_0_RANGE = MIN_FORCE_LEVEL_0_RANGE_12_BIT;
	}

	struct adc_config	adc_conf;
	struct adc_channel_config adcch_conf0;
	struct adc_channel_config adcch_conf1;
	struct adc_channel_config adcch_conf2;
	
	adc_read_configuration(&ADCB, &adc_conf);
	adcch_read_configuration(&ADCB, ADC_CH0, &adcch_conf0);
	adcch_read_configuration(&ADCB, ADC_CH1, &adcch_conf1);
	adcch_read_configuration(&ADCB, ADC_CH2, &adcch_conf2);
	
	adc_set_conversion_parameters(&adc_conf,ADC_SIGN_OFF,ADC_RES_FSR,ADC_REF_VCC);

	adc_set_conversion_trigger(&adc_conf,ADC_TRIG_MANUAL,1,0);
	adc_set_clock_rate(&adc_conf, 200000UL);
	
	adcch_set_input(&adcch_conf0,ADCCH_POS_PIN0,ADCCH_NEG_NONE,1);
	adcch_set_input(&adcch_conf1,ADCCH_POS_PIN1,ADCCH_NEG_NONE,1);
	adcch_set_input(&adcch_conf2,ADCCH_POS_PIN2,ADCCH_NEG_NONE,1);

	adc_write_configuration(&ADCB,&adc_conf);
	adcch_write_configuration(&ADCB,ADC_CH0,&adcch_conf0);
	adcch_write_configuration(&ADCB,ADC_CH1,&adcch_conf1);
	adcch_write_configuration(&ADCB,ADC_CH2,&adcch_conf2);
	adc_enable(&ADCB);

	force_sensors_calibrate(calibrate);


}

void force_sensors_read(uint16_t *results){
	adc_start_conversion(&ADCB,ADC_CH0);
	adc_start_conversion(&ADCB,ADC_CH1);
	adc_start_conversion(&ADCB,ADC_CH2);

	adc_wait_for_interrupt_flag(&ADCB,ADC_CH0);
	adc_wait_for_interrupt_flag(&ADCB,ADC_CH1);
	adc_wait_for_interrupt_flag(&ADCB,ADC_CH2);

	results[0] = adc_get_result(&ADCB,ADC_CH0);
	results[1] = adc_get_result(&ADCB,ADC_CH1);
	results[2] = adc_get_result(&ADCB,ADC_CH2);
}

void force_sensors_read_force_levels(uint8_t *force_levels){
	uint16_t force_reading[NO_OF_SENSORS];
	force_sensors_read(force_reading);
	for (uint8_t sensor_no = 0; sensor_no<NO_OF_SENSORS;sensor_no++){
		for (uint8_t force_level = 0; force_level< NO_OF_FORCE_LEVELS;force_level++){
			if (force_reading[sensor_no]<force_level_limits[sensor_no][force_level]){
				force_levels[sensor_no] = force_level;
				break;
			}
		}
	}

}

void force_sensors_read_force_levels_with_hysteresis(uint8_t *force_levels){
	uint16_t force_reading[NO_OF_SENSORS];
	force_sensors_read(force_reading);

	for (uint16_t sensor_no = 0; sensor_no<NO_OF_SENSORS;sensor_no++){
		for (uint16_t force_level = 0; force_level< NO_OF_FORCE_LEVELS;force_level++){
			if (force_reading[sensor_no]<force_level_limits[sensor_no][force_level]){
				if(force_level && (force_reading[sensor_no] - force_level_limits[sensor_no][force_level]) < HYSTERESIS_WIDTH){
					force_level = previous_force_level[sensor_no];
				}
				force_levels[sensor_no] = force_level;
				previouse_force_reading[sensor_no] = force_reading[sensor_no];
				previous_force_level[sensor_no]  = force_level;
				break;
			}
		}
	}
}