/*
 * magnetic_encoders.c
 *
 * Created: 14/05/2019 09:09:31
 *  Author: jon_e
 */ 



 #include "magnetic_encoders.h"
 
 #define ENCODERS_ENABLE_PIN  J2_PIN3

 #define ANGLE_RANGE_THUMB_PROX 50
 #define ANGLE_RAGNE_THUMB_DIST 90
 #define ANGLE_RANGE_INDEX_PROX 70
 #define ANGLE_RANGE_INDEX_DIST 90

 #define ADC_RES_ENC ADC_RES_8

 #define DEFAULT_MIN_THUMB_PROX_8_BIT 56
 #define DEFAULT_MAX_THUMB_PROX_8_BIT 76
 
 #define DEFAULT_MIN_THUMB_DIST_8_BIT 39
 #define DEFAULT_MAX_THUMB_DIST_8_BIT 74
 
 #define DEFAULT_MIN_INDEX_PROX_8_BIT 89
 #define DEFAULT_MAX_INDEX_PROX_8_BIT 125
 
 #define DEFAULT_MIN_INDEX_DIST_8_BIT 28
 #define DEFAULT_MAX_INDEX_DIST_8_BIT 57

 #define DEFAULT_MIN_THUMB_PROX_12_BIT 905 
 #define DEFAULT_MAX_THUMB_PROX_12_BIT 1214
 
 #define DEFAULT_MIN_THUMB_DIST_12_BIT 632
 #define DEFAULT_MAX_THUMB_DIST_12_BIT 1188
 
 #define DEFAULT_MIN_INDEX_PROX_12_BIT 1387
 #define DEFAULT_MAX_INDEX_PROX_12_BIT 1949
 
 #define DEFAULT_MIN_INDEX_DIST_12_BIT 426
 #define DEFAULT_MAX_INDEX_DIST_12_BIT 901

 


 static uint16_t adc_min_values[NO_OF_ENCODERS];
 static uint16_t adc_max_values[NO_OF_ENCODERS];
 static uint16_t adc_range[NO_OF_ENCODERS];
 static uint16_t angle_range[NO_OF_ENCODERS] = {ANGLE_RANGE_THUMB_PROX,ANGLE_RAGNE_THUMB_DIST,ANGLE_RANGE_INDEX_PROX,ANGLE_RANGE_INDEX_DIST};
 

static void magnetic_encoders_load_min_max_values(void){
		if(ADC_RES_ENC == ADC_RES_8){
			adc_min_values[0] = DEFAULT_MIN_THUMB_PROX_8_BIT;
			adc_min_values[1] = DEFAULT_MIN_THUMB_DIST_8_BIT;
			adc_min_values[2] = DEFAULT_MIN_INDEX_PROX_8_BIT;
			adc_min_values[3] = DEFAULT_MIN_INDEX_DIST_8_BIT;

			adc_max_values[0] = DEFAULT_MAX_THUMB_PROX_8_BIT;
			adc_max_values[1] = DEFAULT_MAX_THUMB_DIST_8_BIT;
			adc_max_values[2] = DEFAULT_MAX_INDEX_PROX_8_BIT;
			adc_max_values[3] = DEFAULT_MAX_INDEX_DIST_8_BIT;
			}else if(ADC_RES_ENC == ADC_RES_12){
			adc_min_values[0] = DEFAULT_MIN_THUMB_PROX_12_BIT;
			adc_min_values[1] = DEFAULT_MIN_THUMB_DIST_12_BIT;
			adc_min_values[2] = DEFAULT_MIN_INDEX_PROX_12_BIT;
			adc_min_values[3] = DEFAULT_MIN_INDEX_DIST_12_BIT;

			adc_max_values[0] = DEFAULT_MAX_THUMB_PROX_12_BIT;
			adc_max_values[1] = DEFAULT_MAX_THUMB_DIST_12_BIT;
			adc_max_values[2] = DEFAULT_MAX_INDEX_PROX_12_BIT;
			adc_max_values[3] = DEFAULT_MAX_INDEX_DIST_12_BIT;
		}
	for (uint8_t sensor_no = 0; sensor_no <NO_OF_ENCODERS;sensor_no++){
		adc_range[sensor_no] = (adc_max_values[sensor_no] - adc_min_values[sensor_no]); // angle_ranges[sensor_no];
	}
}

static void magnetic_encoders_read_raw(int16_t results[NO_OF_ENCODERS]){
	adc_start_conversion(&ADCA,ADC_CH0);
	adc_start_conversion(&ADCA,ADC_CH1);
	adc_start_conversion(&ADCA,ADC_CH2);
	adc_start_conversion(&ADCA,ADC_CH3);

	adc_wait_for_interrupt_flag(&ADCA,ADC_CH0);
	adc_wait_for_interrupt_flag(&ADCA,ADC_CH1);
	adc_wait_for_interrupt_flag(&ADCA,ADC_CH2);
	adc_wait_for_interrupt_flag(&ADCA,ADC_CH3);

	results[0] = adc_get_result(&ADCA,ADC_CH0);
	results[1] = adc_get_result(&ADCA,ADC_CH1);
	results[2] = adc_get_result(&ADCA,ADC_CH2);
	results[3] = adc_get_result(&ADCA,ADC_CH3);

	for (uint8_t encoder_no = 0; encoder_no<NO_OF_ENCODERS;encoder_no++){
		if (results[encoder_no]<adc_min_values[encoder_no]){
			results[encoder_no] = adc_min_values[encoder_no];
			}else if (results[encoder_no]>adc_max_values[encoder_no]){
			results[encoder_no] = adc_max_values[encoder_no];
		}
	}
}








 void magnetic_encoders_init(void){
	struct adc_config adc_conf;
	struct adc_channel_config adcch_conf4;
	struct adc_channel_config adcch_conf5;
	struct adc_channel_config adcch_conf6;
	struct adc_channel_config adcch_conf7;
	
	adc_read_configuration(&ADCA,&adc_conf);
	adcch_read_configuration(&ADCA,ADC_CH0,&adcch_conf4);
	adcch_read_configuration(&ADCA,ADC_CH1,&adcch_conf5);
	adcch_read_configuration(&ADCA,ADC_CH2,&adcch_conf6);
	adcch_read_configuration(&ADCA,ADC_CH3,&adcch_conf7);

	adc_set_conversion_parameters(&adc_conf,ADC_SIGN_OFF,ADC_RES_ENC,ADC_REF_VCC);
	adc_set_conversion_trigger(&adc_conf,ADC_TRIG_MANUAL,1,0);
	adc_set_clock_rate(&adc_conf,200000UL);

	adcch_set_input(&adcch_conf4,ADCCH_POS_PIN4,ADCCH_NEG_NONE,1);
	adcch_set_input(&adcch_conf5,ADCCH_POS_PIN5,ADCCH_NEG_NONE,1);
	adcch_set_input(&adcch_conf6,ADCCH_POS_PIN6,ADCCH_NEG_NONE,1);
	adcch_set_input(&adcch_conf7,ADCCH_POS_PIN7,ADCCH_NEG_NONE,1);

	adc_write_configuration(&ADCA,&adc_conf);
	adcch_write_configuration(&ADCA,ADC_CH0,&adcch_conf4);
	adcch_write_configuration(&ADCA,ADC_CH1,&adcch_conf5);
	adcch_write_configuration(&ADCA,ADC_CH2,&adcch_conf6);
	adcch_write_configuration(&ADCA,ADC_CH3,&adcch_conf7);

	adc_enable(&ADCA);

	magnetic_encoders_load_min_max_values();

	gpio_configure_pin(ENCODERS_ENABLE_PIN,IOPORT_DIR_OUTPUT);

	magnetic_encoders_disable();
 }

 void magnetic_encoders_enable(void){
	gpio_set_pin_high(ENCODERS_ENABLE_PIN);
 }

 void magnetic_encoders_disable(void){
	gpio_set_pin_low(ENCODERS_ENABLE_PIN);
 }




void magnetic_encoders_read_angle(int16_t angles[NO_OF_ENCODERS]){
	int16_t result[NO_OF_ENCODERS];

	magnetic_encoders_read_raw(result);

	for (uint8_t sensor_no = 0; sensor_no< NO_OF_ENCODERS;sensor_no++){
		angles[sensor_no] = ((result[sensor_no]-adc_min_values[sensor_no]) * angle_range[sensor_no])/adc_range[sensor_no];
	}

	//Distal encoder on thumb is mounted such that the min is in the closed position.
	angles[1] = ANGLE_RAGNE_THUMB_DIST - angles[1];
 }