/*
 * vibration_actuators.c
 *
 * Created: 03/05/2019 11:18:44
 *  Author: jon_e
 */
#include "vibration_actuators.h"

 

#define F_SYS 24000000

#define TWI_MASTER       TWIC
#define TWI_MASTER_PORT  PORTC
#define TWI_SPEED        50000
#define TWI_MASTER_ADDR  0x50
#define TWI_SLAVE_ADDR   0x5A

#define DATA_LENGTH 2



twi_options_t m_options = {
	.speed = TWI_SPEED,
	.speed_reg = TWI_BAUD(F_SYS, TWI_SPEED),
	.chip = TWI_MASTER_ADDR
};

uint8_t data[DATA_LENGTH] = {
	0x00, 0x00
};


twi_package_t packet = {
	.addr_length	= 0,
	.chip			= TWI_SLAVE_ADDR,
	.buffer			= (void *)data,
	.length			= DATA_LENGTH,
	.no_wait		= false
};



#define AMPLITUDE0 0
#define AMPLITUDE1 16
#define AMPLITUDE2 32
#define AMPLITUDE3 64
#define AMPLITUDE4 127

uint8_t AMPLITUDES[NO_OF_FORCE_LEVELS+1] ={AMPLITUDE0,AMPLITUDE1,AMPLITUDE2,AMPLITUDE3,AMPLITUDE4};


 static void vibration_actuator_select_actuator(uint8_t actuator_no){
	 if (actuator_no == 0){
		 gpio_configure_pin(J4_PIN1,IOPORT_DIR_OUTPUT);
		 gpio_configure_pin(J4_PIN2,IOPORT_DIR_OUTPUT);
		 gpio_configure_pin(J4_PIN0,IOPORT_DIR_INPUT);
		 gpio_set_pin_high(J4_PIN1);
		 gpio_set_pin_high(J4_PIN2);
		 }else if(actuator_no == 1){
		 gpio_configure_pin(J4_PIN0,IOPORT_DIR_OUTPUT);
		 gpio_configure_pin(J4_PIN2,IOPORT_DIR_OUTPUT);
		 gpio_configure_pin(J4_PIN1,IOPORT_DIR_INPUT);
		 gpio_set_pin_high(J4_PIN0);
		 gpio_set_pin_high(J4_PIN2);
		 }else if(actuator_no == 2){
		 gpio_configure_pin(J4_PIN0,IOPORT_DIR_OUTPUT);
		 gpio_configure_pin(J4_PIN1,IOPORT_DIR_OUTPUT);
		 gpio_configure_pin(J4_PIN2,IOPORT_DIR_INPUT);
		 gpio_set_pin_high(J4_PIN0);
		 gpio_set_pin_high(J4_PIN1);
		 }else{
		 gpio_configure_pin(J4_PIN0,IOPORT_DIR_INPUT);
		 gpio_configure_pin(J4_PIN1,IOPORT_DIR_INPUT);
		 gpio_configure_pin(J4_PIN2,IOPORT_DIR_INPUT);
	 }
 }

 static status_code_t vibration_actuator_autocalibrate(uint8_t actuator_no){
	 vibration_actuator_select_actuator(actuator_no);
	 delay_ms(1);

	 status_code_t twi_send_status;

	 data[0] = 0x01;
	 data[1] = 0x00;
	 twi_send_status = twi_master_write(&TWIC,&packet);


	 data[0] = 0x1A;		//	Setting LRA, FB_BRAKE_FACTOR, LOOP_GAIN
	 data[1] = 0b10110110;
	 twi_send_status += twi_master_write(&TWIC,&packet);

	 data[0] = 0x16;	//Rated voltage see eq 5 in data sheet for driver
	 data[1] = 0x3a;	//Calculated 5A
	 twi_send_status += twi_master_write(&TWIC,&packet);

	 data[0] = 0x17;	//Setting OD_CLAMP
	 data[1] = 0x76;
	 twi_send_status += twi_master_write(&TWIC,&packet);

	 data[0] = 0x1E;
	 data[1] = 0b00110000;
	 twi_send_status += twi_master_write(&TWIC,&packet);

	 data[0] = 0x0C;	//Setting the GO-bit starting the procedure
	 data[1] = 0x1;

	 twi_send_status += twi_master_write(&TWIC,&packet);

	 delay_ms(100);

	 data[0] = 0x01;
	 data[1] = 0b101;

	 twi_send_status += twi_master_write(&TWIC,&packet);

	 return twi_send_status;
 }

 static void vibration_actuator_set_amplitude(uint8_t amplitude){
	 data[0] = 0x02;
	 data[1] = amplitude;

	 twi_master_write(&TWIC,&packet);
 }
 

 void vibration_actuator_set_amplitude_level(uint8_t actuator_no,uint8_t level){
	vibration_actuator_select_actuator(actuator_no);
	vibration_actuator_set_amplitude(AMPLITUDES[level]);
 }

 void vibration_actuator_all_off(void){
	for(uint8_t actuator_no = 0; actuator_no< NO_OF_ACTUATORS;actuator_no++){
		vibration_actuator_set_amplitude_level(actuator_no,0);
	}
 }

 status_code_t vibration_actuator_init(void){
	 sysclk_enable_peripheral_clock(&TWI_MASTER);
	 twi_master_init(&TWIC,&m_options);
	 twi_master_enable(&TWIC);

	 for (uint8_t actuator = 0;actuator<NO_OF_ACTUATORS;actuator++){
		 uint8_t res = vibration_actuator_autocalibrate(actuator);
		 if (res!= 0){
			 return res;
		 }
	 }
	 return 0;
 }